#!/usr/bin/env python
# coding: utf-8

from datetime import datetime
import os

import matplotlib.pyplot as plt
import numpy as np
import xarray

import dplume

"""
Make example plots for performance of plume detection algorithm used in
Kuhlmann et al. (2019)
"""


PATH = '/project/kug/smartcarb/results/Berlin/figures'


def main(time, tracer, ssp, ns=37, orbit=None, lon_eq=None, make_figure=True,
         fig_height=None, fig_width=None, xlim=None, ylim=None, vlim=(-2,2),
         legend_loc='upper right'):

    q = 0.99
    nsat = 6


    # load results for 20 samples
    ts = dplume.io.read_results(tracer, ssp, ns=ns, q=q, nsat=nsat,
                                origin='Berlin')
    ts = dplume.io.filter_overlap(ts, nmax=100)

    ts = ts[ts.index == time]
    px = ts.detected_pixels.values
    print(px)
    TP = ts.CO2_TP.values
    PPV = ts.CO2_PPV.values

    if len(ts) != 20:
        raise RuntimeError("The data should have 20 samples.")

    print(tracer, ssp, ns)
    print('detect:\t %.0f +/- %.0f' % (px.mean(), px.std()))
    print('TP:\t %.0f +/- %.0f' % (TP.mean(), TP.std()))
    print('PPV:\t %.2f +/- %.2f' % (np.nanmean(PPV), np.nanstd(PPV)))
    print()


    if not make_figure:
        return


    for i in [#np.argmin(px),
              np.argmin(np.abs(np.median(px) - px)),
              #np.argmax(px)
             ]:

        filenames = dplume.io.get_sample_filenames(time, tracer, ssp,
                                                   nsample=i, ns=ns,
                                                   orbit=orbit, lon_eq=lon_eq)
        filename = filenames[0]
        #print(i, os.path.basename(filename), len(filenames))

        data = xarray.open_dataset(filename)

        if ssp == 'S5':
            print(filename)
            data7 = dplume.io.read_s7_for_s5_result(filename, 'Berlin')
        else:
            data7 = None

        filename = os.path.basename(filename).replace('.nc', '_v2.png')
        filename = os.path.join(PATH, filename)

        fig = dplume.vis.visualize(data, data7=data7, legend_loc=legend_loc,
                                   is_sentinel_5=(ssp=='S5'), vlim=vlim)
        ax, _ = fig.get_axes()

        if fig_height is not None:
            fig.set_figheight(fig_height)

        if fig_width is not None:
            fig.set_figwidth(fig_width)

        if xlim is not None:
            ax.set_xlim(*xlim)

        if ylim is not None:
            ax.set_ylim(*ylim)

        plt.savefig(filename)
        plt.savefig(filename.replace('.png', '.pdf'))



if __name__ == '__main__':
    make_figure = True
    legend_loc = 'upper right'

    # CO2: weak signal because of wind speed
    if False:
        time = datetime(2015,7,2,11)
        orbit = 2714
        lon_eq = 1207

        fig_height = 3.85
        fig_width = 7.0
        xlim = (0.45, 2.3)
        ylim = (5.2, 6.4)

        vlim = (-2,2)

        cases = [
            ('CO2', 0.5, 37),
            ('NO2', 'high', 37)
        ]
    # CO2 gradient misfit
    if False:
        time = datetime(2015,2,27,11)
        orbit = 850
        lon_eq = 401

        fig_height = 3.85
        fig_width = 7.0
        xlim = (1.2, 2.9)
        ylim = (5.4, 6.5)

        vlim = (-2,2)

        cases = [
            ('CO2', 0.5, 37),
            ('NO2', 'high', 37)
        ]

    # Sentinel-7 cloud-free but Sentinel-5 cloudy
    if False:
        time = datetime(2015,12,3,11)
        orbit = 5010
        lon_eq = 1207

        fig_height = 3.85
        fig_width = 7.0
        xlim = (1.7, 3.1)
        ylim = (5.4, 6.3)

        vlim = (-2,2)
        legend_loc = 'upper left'

        cases = [
            ('NO2', 'high', 37),
            ('NO2', 'S5', 5)
        ]


    # Sentinel-7: plume shift
    if True:
        time = datetime(2015,6,17,11)
        orbit = 2490
        lon_eq = 401

        fig_height = 3.85
        fig_width = 7.0
        xlim = (1.8, 3.5)
        ylim = (4.8, 5.9)

        vlim = (-2,2)
        legend_loc = 'upper right'

        cases = [
            ('NO2', 'high', 37),
            ('NO2', 'S5', 5)
        ]

    # case: overview
    if False:
        time = datetime(2015,4,21,11)
        orbit = 1640
        lon_eq = 0

        fig_height = 3.85
        fig_width = 7.0
        xlim = (1.86, 3.42)
        ylim = (4.74, 5.76)

        vlim = (-3,3)

        cases = [
            ('CO2', 0.5, 37),
            #('CO2', 0.7, 37),
            ('CO2', 1.0, 37),
            #('NO2', 'low', 37),
            ('NO2', 'high', 37),
            ('NO2', 'S5', 5),
        ]


    for tracer, ssp, ns in cases:
        main(time, tracer, ssp, ns, make_figure=make_figure,
             lon_eq=lon_eq, orbit=orbit, fig_height=fig_height,
             fig_width=fig_width, xlim=xlim, ylim=ylim, vlim=vlim,
             legend_loc=legend_loc)




