#/!usr/bin/env python
# coding: utf-8

from datetime import datetime
import glob
import os
import multiprocessing
import sys

import matplotlib
import numpy as np
import xarray

import amrs.models
import omi

import smartcarb.config as cfg
import smartcarb.io



def mask_relaxation_zone(grid, size=30):
    grid['values'][:size,:] = np.nan
    grid['values'][:,:size] = np.nan
    grid['values'][:,-size:] = np.nan
    grid['values'][-size:,:] = np.nan

    return grid




def main(filenames, tracer='NO2', domain='smartcarb', attrs=None):
    """
    Grid Tropomi NO2 fields.
    """
    if isinstance(domain, str):
        domain = cfg.DOMAINS[domain]

    # read Level-2 data and create Level-3 grid
    grid = omi.create_grid(domain.startlon, domain.startlat, domain.stoplon,
                           domain.stoplat, domain.dlon, domain.dlat,
                           pollon=domain.pollon, pollat=domain.pollat)

    # global attributes
    if attrs is None:
        attrs = {}

    attrs['title'] = 'CO2M NO2 (SMARTCARB project)'

    attrs['creator_name'] = 'Gerrit Kuhlmann'
    attrs['creator_email'] = 'gerrit.kuhlmann@empa.ch'
    attrs['institution'] = ('Swiss Federal Laboratories for Materials Science '
                            'and Technology (Empa)')

    attrs['references'] = 'OMI Gridding Algorithm (doi: 10.5194/amt-7-451-2014)'
    attrs['date_created'] = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')



    for filename in filenames:
        print(filename)

        level2 = smartcarb.io.read_level2(filename, co2_noise=0.7, co2_thr=0.01,
                                          no2_noise='high', no2_thr=0.30)

        # remove data
        mask = omi.mask_grid_domain(grid, level2.lonc.values, level2.latc.values)

        if not np.any(mask):
            return grid

        # Prepare values, errors and weigths
        rho = np.array(level2[tracer])
        errors = np.ones_like(rho)
        weights = np.ones_like(rho)
        missing_values = np.array(level2.clouds > 0.30)

        # swap axes and directions
        lonc = np.swapaxes((level2['lonc']).T, 1,2)[::-1]
        latc = np.swapaxes((level2['latc']).T, 1,2)[::-1]

        # grid data
        grid = omi.cvm_grid(grid, lonc.values, latc.values,
                            rho, errors, weights, missing_values)

    # weight observations
    grid['values'][:] = grid['values'] / grid['weights']

    # update grid field attributes using Level-2 data


    # set model relaxation zone to nans (30 grid cells)
    grid = mask_relaxtion_zone(grid, size=30)

    del grid['values']
    del grid['weights']
    del grid['errors']

    # add attributes
    mean_time_utc = data['time_utc'].astype('int64').mean().astype('datetime64[ns]').values
    grid.attrs['mean_time_utc'] = str(mean_time_utc)

    # save to netCDF4 file
    nc_filename = os.path.join(output_path, 'S5P_NO2_%s_o%05d.nc' %
                               (data.time_reference, data.orbit))
    grid.to_netcdf(nc_filename)

    # make image showing NO2 field
    fig = make_figure(grid, mean_time_utc, grid.orbit, domain)
    image_filename = os.path.join(output_path, 'S5P_NO2_%s_o%05d.png' %
                                  (data.time_reference, data.orbit))
    fig.savefig(image_filename)

    #return data, grid



if __name__ == '__main__':

    #number, n_cpu = int(sys.argv[1]), int(sys.argv[2])

    filenames = sorted(glob.glob(os.path.join(PATH, '*.nc')))#[number::n_cpu]
    output_path = '/scratch/snx3000/gkuhl/S5P'

    with multiprocessing.Pool(processes=15) as pool:
        pool.starmap(main, [(fn, output_path, 0.5, DOMAIN_ALPINE) for fn in
                            filenames])

    #for fn in filenames:
    #    main(fn, output_path, 0.5, DOMAIN_ALPINE)

